@@ -3,6 +3,7 @@ package WWW::CDBaby;
 use WWW::Sitebase::Navigator -Base;
 use warnings;
 use strict;
+use Carp;
 
 our $DEBUG = 0;
 
@@ -12,11 +13,11 @@ WWW::CDBaby - Automate interaction with cdbaby.com!
 
 =head1 VERSION
 
-Version 0.01
+Version 0.03
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -33,6 +34,7 @@ field site_info => {
        account_field => 'login_username', # Fieldname from the login form
        password_field => 'login_password', # Password fieldname
        cache_dir => '.www-cdbaby',
+       login_form_no => 1,
        login_verify_re => 'Members Login HOME', # (optional)
                # Non-case-sensitive RE we should see once we're logged in
        not_logged_in_re => 'LOG IN to your CD Baby',
@@ -82,7 +84,7 @@ file" link.  See how much easier this method is? ;-)
 
     my $cdbaby = new WWW::CDBaby;
 
-    my ( @sales ) = $cdbaby->get_sales( $album_id );
+    my ( @sales ) = $cdbaby->get_cd_sales( $album_id );
 
     my $total=0;
     foreach $sale ( @sales ) {
@@ -96,7 +98,9 @@ file" link.  See how much easier this method is? ;-)
 sub get_cd_sales {
 
     my ( $album_id ) = @_;
-    
+
+    croak "Must pass an album ID to get_cd_sales" unless ( $album_id );
+
     my $res = $self->get_page( "http://members.cdbaby.com/show_sales_download/$album_id.txt" )
         or return;
     
@@ -142,6 +146,123 @@ sub get_cd_sales {
 
 }
 
+=head2 C<get_dd_sales( album_id )>
+
+Pass this method the URL of your album.  If you can see your
+album at "cdbaby.com/amberg", your album_id is "amberg".
+
+It returns an array of hashrefs containing your digital distribution
+sales and plays for that album.
+
+This method gets the HTML table you get if you Go to the "Digital" tab
+and click the amount next to INCOME for one of your albums.  It parses
+the HTML into one hash for each row.  The keys to the hash are taken
+directly from the headers at the top of the table and modified to make
+them program-friendly:
+
+ Leading and trailing whitespace is stripped
+ white space is replaced by "_"
+ # by itself is turned into "quantity"
+ caps are made lower case.
+ Any remaining characters that aren't letters, numbers, or _ are stripped
+
+The current keys returned (as of 8/1/2007) are:
+
+ company
+ sales_date
+ report_date
+ song
+ price
+ quantity
+ subtotal
+
+As these keys are taken directly from the headers at the top of the table,
+if rows are added or removed or the headers are changed by CD Baby,
+the keys to your hash will change accordingly.
+
+Also, the dollar sign ("$") from the price fields is removed so you can
+do things like the example below:
+
+    use WWW::CDBaby;
+
+    my $cdbaby = new WWW::CDBaby;
+
+    my ( @sales ) = $cdbaby->get_dd_sales( $album_id );
+
+    my $total=0;
+    foreach $sale ( @sales ) {
+        $total += $sale->{'subtotal'};
+    }
+    
+    print "Total profits: \$$total\n";
+
+(Note: when I run this script, I get a number slightly lower than
+the total shown on the DD page.  This is probably either CD Baby rounding
+the numbers (probably up :) or some floating point issue.)
+
+=cut
+
+sub get_dd_sales {
+
+    my ( $album_id ) = @_;
+    
+    croak "Must pass an album ID to get_dd_sales" unless ( $album_id );
+    
+    my @fieldnames = ();
+    my @field_arr = ();
+
+    ( $DEBUG ) && print "Getting sales page\n";
+    my $res = $self->get_page( "https://members.cdbaby.com/accounting?view=ddalbum&a=$album_id" )
+        or return;
+
+    ( $DEBUG ) && print "Parsing headers...\n";
+    # Get the headers
+    my $page = $res->decoded_content;
+
+    while ( $page =~ s/.*?<th>(<a .*?>)?(.*?)<//ismo ) {
+        my $fn = $2;
+        $fn =~ s/^\s*(.*)\s*$/$1/; # Strip trailing & leading whitespace
+        $fn =~ s/\s+/_/g;          # Turn whitespace into _
+        $fn =~ s/^#$/no/g;         # Turn a lone "#" into "quantity"
+        $fn = lc( $fn );           # Make it all lower case
+        $fn =~ s/[^a-z0-9_]//g;    # Strip anything else
+        push ( @fieldnames, "$fn" );
+        ( $DEBUG ) && print "header: $fn\n";
+    }
+    ($DEBUG) && print "\n\nParsing sales...\n";
+
+    # Get the rows
+    # We ignore the row definitions and just loop through TD tags, applying
+    # each one in sequence to the field names in @fieldnames.  When we
+    # get to the end of @fieldnames, we store that hash in @field_arr and go on
+    # to the next row.
+    my $count=0;
+    my %row = ();
+    while ( $page =~ s/.*?<td>(<a .*?>)?(.*?)<\/td>//ismo ) {
+        my $fv = $2;
+        $fv =~ s/^\s*(.*)\s*$/$1/; # Strip trailing & leading whitespace
+        $fv =~ s/<.*?>//g;  # Strip links
+        $fv =~ s/^\$//;     # Strip leading dollar sign (for prices)
+        $row{"$fieldnames[$count]"}="$fv";
+        ($DEBUG) && print "$fieldnames[$count]: $fv\n";
+        $count++;
+
+        if ( $count >= @fieldnames ) {
+            # Store the row
+            push( @field_arr, { %row } );
+
+            # Reset the counter
+            $count=0;
+            %row=();
+            
+            ($DEBUG) && print "\n\n";
+        }
+    }
+    
+    return ( @field_arr );
+    
+}
+
 =head1 AUTHOR
 
 Grant Grueninger, C<< <grantg at cpan.org> >>
